/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM ../../../dist/idl/nsIAtom.idl
 */

#ifndef __gen_nsIAtom_h__
#define __gen_nsIAtom_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
#include "nsStringGlue.h"
#include "nsCOMPtr.h"
#include "nsStringBuffer.h"

/* starting interface:    nsIAtom */
#define NS_IATOM_IID_STR "8b8c11d4-3ed5-4079-8974-73c7576cdb34"

#define NS_IATOM_IID \
  {0x8b8c11d4, 0x3ed5, 0x4079, \
    { 0x89, 0x74, 0x73, 0xc7, 0x57, 0x6c, 0xdb, 0x34 }}

class nsIAtom : public nsISupports {
 public:

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IATOM_IID)

  /* [binaryname(ScriptableToString)] AString toString (); */
  NS_IMETHOD ScriptableToString(nsAString & _retval) = 0;

  /* [noscript] AUTF8String toUTF8String (); */
  NS_IMETHOD ToUTF8String(nsACString & _retval) = 0;

  /* [binaryname(ScriptableEquals)] boolean equals (in AString aString); */
  NS_IMETHOD ScriptableEquals(const nsAString & aString, bool *_retval) = 0;

  /* [noscript,notxpcom] size_t SizeOfIncludingThis (in MallocSizeOf aMallocSizeOf); */
  NS_IMETHOD_(size_t) SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf) = 0;

   // note this is NOT virtual so this won't muck with the vtable!
  inline bool Equals(const nsAString& aString) const {
    return aString.Equals(nsDependentString(mString, mLength));
  }
  inline bool IsStaticAtom() const {
    return mIsStatic;
  }
  inline char16ptr_t GetUTF16String() const {
    return mString;
  }
  inline uint32_t GetLength() const {
    return mLength;
  }
  inline void ToString(nsAString& aBuf) {
    // See the comment on |mString|'s declaration.
    nsStringBuffer::FromData(mString)->ToString(mLength, aBuf);
  }
  inline nsStringBuffer* GetStringBuffer() const {
    // See the comment on |mString|'s declaration.
    return nsStringBuffer::FromData(mString);
  }
  /**
   * A hashcode that is better distributed than the actual atom
   * pointer, for use in situations that need a well-distributed
   * hashcode.
   */
  inline uint32_t hash() const {
    return mHash;
  }
protected:
  uint32_t mLength:31;
  uint32_t mIsStatic:1;
  uint32_t mHash;
  /**
   * WARNING! There is an invisible constraint on |mString|: the chars it
   * points to must belong to an nsStringBuffer. This is so that the
   * nsStringBuffer::FromData() calls above are valid.
   */
  char16_t* mString;
};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIAtom, NS_IATOM_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIATOM \
  NS_IMETHOD ScriptableToString(nsAString & _retval) override; \
  NS_IMETHOD ToUTF8String(nsACString & _retval) override; \
  NS_IMETHOD ScriptableEquals(const nsAString & aString, bool *_retval) override; \
  NS_IMETHOD_(size_t) SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf) override; \

/* Use this macro when declaring the members of this interface when the
   class doesn't implement the interface. This is useful for forwarding. */
#define NS_DECL_NON_VIRTUAL_NSIATOM \
  NS_METHOD ScriptableToString(nsAString & _retval); \
  NS_METHOD ToUTF8String(nsACString & _retval); \
  NS_METHOD ScriptableEquals(const nsAString & aString, bool *_retval); \
  NS_METHOD_(size_t) SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf); \

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIATOM(_to) \
  NS_IMETHOD ScriptableToString(nsAString & _retval) override { return _to ScriptableToString(_retval); } \
  NS_IMETHOD ToUTF8String(nsACString & _retval) override { return _to ToUTF8String(_retval); } \
  NS_IMETHOD ScriptableEquals(const nsAString & aString, bool *_retval) override { return _to ScriptableEquals(aString, _retval); } \
  NS_IMETHOD_(size_t) SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf) override { return _to SizeOfIncludingThis(aMallocSizeOf); } \

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIATOM(_to) \
  NS_IMETHOD ScriptableToString(nsAString & _retval) override { return !_to ? NS_ERROR_NULL_POINTER : _to->ScriptableToString(_retval); } \
  NS_IMETHOD ToUTF8String(nsACString & _retval) override { return !_to ? NS_ERROR_NULL_POINTER : _to->ToUTF8String(_retval); } \
  NS_IMETHOD ScriptableEquals(const nsAString & aString, bool *_retval) override { return !_to ? NS_ERROR_NULL_POINTER : _to->ScriptableEquals(aString, _retval); } \
  NS_IMETHOD_(size_t) SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf) override; \

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsAtom : public nsIAtom
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIATOM

  nsAtom();

private:
  ~nsAtom();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS(nsAtom, nsIAtom)

nsAtom::nsAtom()
{
  /* member initializers and constructor code */
}

nsAtom::~nsAtom()
{
  /* destructor code */
}

/* [binaryname(ScriptableToString)] AString toString (); */
NS_IMETHODIMP nsAtom::ScriptableToString(nsAString & _retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* [noscript] AUTF8String toUTF8String (); */
NS_IMETHODIMP nsAtom::ToUTF8String(nsACString & _retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* [binaryname(ScriptableEquals)] boolean equals (in AString aString); */
NS_IMETHODIMP nsAtom::ScriptableEquals(const nsAString & aString, bool *_retval)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* [noscript,notxpcom] size_t SizeOfIncludingThis (in MallocSizeOf aMallocSizeOf); */
NS_IMETHODIMP_(size_t) nsAtom::SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif

/*
 * The four forms of NS_Atomize (for use with |nsCOMPtr<nsIAtom>|) return the
 * atom for the string given. At any given time there will always be one atom
 * representing a given string. Atoms are intended to make string comparison
 * cheaper by simplifying it to pointer equality. A pointer to the atom that
 * does not own a reference is not guaranteed to be valid.
 */
/**
 * Find an atom that matches the given UTF-8 string.
 * The string is assumed to be zero terminated.  Never returns null.
 */
extern already_AddRefed<nsIAtom> NS_Atomize(const char* aUTF8String);
/**
 * Find an atom that matches the given UTF-8 string.  Never returns null.
 */
extern already_AddRefed<nsIAtom> NS_Atomize(const nsACString& aUTF8String);
/**
 * Find an atom that matches the given UTF-16 string.
 * The string is assumed to be zero terminated.  Never returns null.
 */
extern already_AddRefed<nsIAtom> NS_Atomize(const char16_t* aUTF16String);
/**
 * Find an atom that matches the given UTF-16 string.  Never returns null.
 */
extern already_AddRefed<nsIAtom> NS_Atomize(const nsAString& aUTF16String);
/**
 * Return a count of the total number of atoms currently
 * alive in the system.
 */
extern nsrefcnt NS_GetNumberOfAtoms(void);
/**
 * Return a pointer for a static atom for the string or null if there's 
 * no static atom for this string.
 */
extern nsIAtom* NS_GetStaticAtom(const nsAString& aUTF16String);
/**
 * Seal the static atom table
 */
extern void NS_SealStaticAtomTable();
class nsAtomString : public nsString
{
public:
  explicit nsAtomString(nsIAtom* aAtom)
  {
    aAtom->ToString(*this);
  }
};
class nsAtomCString : public nsCString
{
public:
  explicit nsAtomCString(nsIAtom* aAtom)
  {
    aAtom->ToUTF8String(*this);
  }
};
class nsDependentAtomString : public nsDependentString
{
public:
  explicit nsDependentAtomString(nsIAtom* aAtom)
    : nsDependentString(aAtom->GetUTF16String(), aAtom->GetLength())
  {
  }
};

#endif /* __gen_nsIAtom_h__ */
