#define MOZ_SOURCE_STAMP 416dc3163a1f27b8783ed14660d1b373e830df2f
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-release
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-release/rev/416dc3163a1f27b8783ed14660d1b373e830df2f
